/*////////////////////////////////////////////////////////////////////////
Copyright (c) 1994 Electrotechnical Laboratry (ETL), AIST, MITI

Permission to use, copy, modify, and distribute this material for any
purpose and without fee is hereby granted, provided that the above
copyright notice and this permission notice appear in all copies, and
that the name of ETL not be used in advertising or publicity pertaining
to this material without the specific, prior written permission of an
authorized representative of ETL.
ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY OF THIS
MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", WITHOUT ANY EXPRESS
OR IMPLIED WARRANTIES.
/////////////////////////////////////////////////////////////////////////
Content-Type:	program/C; charset=US-ASCII
Program:	file.c
Author:		Yutaka Sato <ysato@etl.go.jp>
Description:
History:
	951029	extracted from DelaGate
//////////////////////////////////////////////////////////////////////#*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
extern FILE *tmpfile();
#include <string.h>
extern char *strdup();
#include <ctype.h>

#define RWXmode (S_IREAD | S_IWRITE | S_IEXEC)
int access_RWX(path)
	char *path;
{	struct stat st;

	if( stat(path,&st) == 0 )
		if( (st.st_mode & RWXmode) == RWXmode )
			return 0;
	return -1;
}

int File_mtime(path)
	char *path;
{	struct stat st;

	if( stat(path,&st) == 0 )
		return st.st_mtime;
	else	return -1;
}
int File_size(path)
	char *path;
{	struct stat st;

	if( stat(path,&st) == 0 )
		return st.st_size;
	else	return -1;
}
int File_ino(path)
	char *path;
{	struct stat st;

	if( stat(path,&st) == 0 )
		return st.st_ino;
	else	return 0;
}
File_cmp(path1,path2)
	char *path1,*path2;
{	struct stat st1,st2;
	static struct stat st0;

	st1 = st0;
	st2 = st0;
	if( stat(path1,&st1) != 0 ) return -1;
	if( stat(path2,&st2) != 0 ) return -1;

/* st_dev will differ from each other for NFS files mounted by
 * different host name (even when they are same in IP address)
 * (/net/host/path != /net/alias/path)
 */
	if( st1.st_dev == st2.st_dev &&
	    st1.st_ino == st2.st_ino )
		return 0;
	else	return 1;
/*
return bcmp(&st1,&st2,sizeof(struct stat));
*/
}

fcompare(fp1,fp2)
	FILE *fp1,*fp2;
{	char buf1[1024],buf2[1024];
	int off1,off2;
	int rc1,rc2,ri;
	int diff;

	off1 = ftell(fp1);
	off2 = ftell(fp2);
	diff = 0;
	for(;;){
		rc1 = fread(buf1,1,1024,fp1);
		rc2 = fread(buf2,1,1024,fp2);
		if( rc1 != rc2 ){
			diff = 1;
			break;
		}
		if( rc1 == 0 )
			break;
		for( ri = 0; ri < rc1; ri++ )
			if( buf1[ri] != buf2[ri] ){
				diff = 1;
				break;
			}
	}
	fseek(fp1,off1,0);
	fseek(fp2,off2,0);
	return diff;
}

Ftruncate(fp,offset,whence)
	FILE *fp;
{	long savoff;
	int rcode;

	savoff = ftell(fp);
	fseek(fp,offset,whence);
	rcode = ftruncate(fileno(fp),(off_t)ftell(fp));
	fseek(fp,savoff,0);
	return rcode;
}
static char *fullpath(path,buff)
	char *path,*buff;
{
	if( !IsWindows() )
		return path;

	/*
	 * //X/path -- OpenNT -- X:/path
	 */ 
	if( path[0] == '/' && path[1] == '/'
	 && 'A' <= path[2] && path[2] <= 'Z'
	 && path[3] == '/' )
	{
		buff[0] = path[2];
		buff[1] = ':';
		strcpy(&buff[2],&path[3]);
		path = buff;
	}
	return path;
}
isFullpath(path)
	char *path;
{
	if( IsWindows() ){
		if( isalpha(path[0]) && path[1] == ':' ){
			if( path[2] == '\\' || path[2] == '/' )
				return 3;
			else	return 2;
		}
		if( path[0] == '\\' )
			return 1;
	}
	if( path[0] == '/' )
		return 1;
	return 0;
}
isBoundpath(path)
	char *path;
{
	if( isFullpath(path) )
		return 1;
	if( path[0] == '.' ){
		if( IsWindows() && path[1] == '\\' )
			return 1;
		if( path[1] == '/' )
			return 1;
	}
	return 0;
}
chdir_cwd(cwd, go, userdir)
	char *cwd,*go;
{	char *tp,*cp;
	int uplen,len;

	for( tp = cwd; *tp; tp++ ){}
	if( len = isFullpath(go) ){
		tp = cwd;
		while( 0 < len-- )
			*tp++ = *go++;
	}else
	if( userdir && go[0] == '~' ){
		if( go[1] == 0   ){ tp = cwd; go += 1; }else
		if( go[1] == '/' ){ tp = cwd; go += 2; }
	}

	while( *go ){
		uplen = 0;
		if( go[0] == '.' ){
			if( go[1] == 0   ){ go += 1; break; }
			if( go[1] == '/' ){ go += 2; continue; }
			if( go[1] == '.' ){
				if( go[2] == 0   ){ uplen = 2; } else
				if( go[2] == '/' ){ uplen = 3; }
			}
		}
		if( uplen ){
			if( cwd < tp ){
				while( cwd+1 < tp ){
					if( tp[-1] == '/' )
						tp--;
					else	break;
				}
				*tp = 0;
				if( cp = strrchr(cwd,'/') ){
					if( cp == cwd  )
						tp = cp + 1;
					else	tp = cp;
				}else	tp = cwd;
			}
			go += uplen;
		}else{
			if( cp = strchr(go,'/') )
				len = (cp - go) + 1;
			else	len = strlen(go);

			if( cwd < tp && tp[-1] != '/' )
				*tp++ = '/';
			strncpy(tp,go,len);
			tp += len;
			go += len;
		}
	}
	while( cwd+1 < tp ){
		if( tp[-1] == '/' )
			tp--;
		else	break;
	}
	*tp = 0;
}

#ifdef _MSC_VER
static char *_TMPDIR = "\\tmp";
#else
static char *_TMPDIR;
#endif
static int _TMPDIR_set;
static int _TMPSEQ;

setTMPDIR(dir)
	char *dir;
{	char buff[1024];

	if( dir ){
		if( _TMPDIR && _TMPDIR_set )
			free(_TMPDIR);
		_TMPDIR = strdup(fullpath(dir,buff));
		_TMPDIR_set = 1;
	}
}

extern FILE *fopentmpfile();
FILE *TMPFILE(what)
	char *what;
{	FILE *fp;
	int besilent;
	char path[1024],buff[1024];

	fp = NULL;
	if( _TMPDIR ){
		mkdir(_TMPDIR,0777);
		sprintf(path,"%s/dg%d.%d.%d",_TMPDIR,getpid(),++_TMPSEQ,
			time(0));
		fp = fopentmpfile(fullpath(path,buff));
		if( fp != NULL )
			syslog_DEBUG("TMPFILE = (%d) %s\n",fileno(fp),path);
		else	syslog_ERROR("TMPFILE: cannot create = %s\n",path);
	}
	if( fp == NULL )
		fp = tmpfile();

	besilent = what[0] == '-';

	if( fp != NULL ){
		setCloseOnExec(fileno(fp));
		if( !besilent )
		syslog_DEBUG(">>>>TMPFILE(%s)>>>>%x[%d]\n",what,fp,fileno(fp));
	}else{
		syslog_ERROR(">>>>TMPFILE(%s)>>>> cannot create.\n",what);
		exit(0);
	}
	return fp;
}

FILE *Tmpfile()
{
	return TMPFILE("Tmpfile");
}

#ifdef NULLFP
#undef NULLFP /* defined in <sys/file.h> of Solaris */
#endif

FILE *NULLFP(){
	static FILE *NULLFP;
	FILE *tmp;

	if( NULLFP == NULL ){
		tmp = TMPFILE("NULLFP");
		/*close(fileno(NULLFP)); should hold to avoid reuse */
		NULLFP = fdopen(fileno(tmp),"r");
	}
	return NULLFP;
}

copyfile2(sfp,dfp,doflush)
	FILE *sfp,*dfp;
{	int buff[1024];
	int totalc,rcc;

	totalc = 0;
	for(;;){
		rcc = fread(buff,1,sizeof(buff),sfp);
		if( rcc == 0 )
			break;
		totalc += rcc;
		if( fwrite(buff,1,rcc,dfp) == 0 )
			break;
	}
	if( doflush )
		fflush(dfp);
	return totalc;
}

copyfile1(sfp,dfp)
	FILE *sfp,*dfp;
{
	return copyfile2(sfp,dfp,1);
}


reads(fd,buf,len)
	char *buf;
{	int rcc,rc1;

	rcc = 0;

	/* touch the I/O buffer before use,
	 * to confirm stack allocation of the buffer in Solaris2.X cc -O ? */
	buf[0] = 0;

	while( rcc < len ){
		rc1 = read(fd,buf+rcc,len-rcc);
		if( rc1 <= 0 )
			break;
		rcc += rc1;
	}
	return rcc;
}


extern char *malloc();
extern char *realloc();

char *Malloc(ptr,size)
	char *ptr;
{	char *nptr;

	if( ptr == NULL )
		nptr = malloc(size);
	else	nptr = realloc(ptr,size);
	if( nptr == NULL ){
		fprintf(stderr,"[%d] DeleGate: Malloc() failed\n",getpid());
		exit(1);
	}
	return nptr;
}
